/* ginger.c */
/* ginger-monaka specific routines */

#define USE_16BIT_FTABLE
#ifndef INITIAL_CONFIG
#define INITIAL_CONFIG 7
#endif

struct portConfig_t {
  uchar ddrbIn;			/* DDRB 入力ピンのビット */
  uchar ddrbOut;		/* DDRB 出力ピンのビット */
  uchar ddrcIn;			/* DDRC 入力ピンのビット */
  uchar ddrcOut;		/* DDRC 出力ピンのビット */
  uchar ddrdIn;			/* DDRD 入力ピンのビット */
  uchar ddrdOut;		/* DDRD 出力ピンのビット */
} portConfig [] = {
  { 0x10, 0x2f, 0x00, 0x3f, 0x00, 0xfa}, /* Config 0 */
  { 0x10, 0x2f, 0x00, 0x3f, 0x00, 0xfa}, /* Config 1 */
  { 0x10, 0x2f, 0x00, 0x3f, 0x00, 0xfa}, /* Config 2 */
  { 0x1c, 0x23, 0x00, 0x3f, 0x28, 0xd2}, /* Config 3 */
  { 0x10, 0x2f, 0x00, 0x3f, 0x00, 0xfa}, /* Config 4 */
  { 0x1d, 0x22, 0x3f, 0x00, 0xfa, 0x00}, /* Config 5 */
  { 0x1f, 0x20, 0x3f, 0x00, 0xfa, 0x00}, /* Config 6 */
  { 0x1f, 0x20, 0x3f, 0x00, 0xfa, 0x00}, /* Config 7 */
};

struct portInfo_t {
  uchar tone_no;
  volatile uchar *port;
  uchar bit;
  uchar status;
};

/* mapping table for Output port and note */
struct portInfo_t portInfoOut []= {
  {60, &PORTC, 0, 0},		/* C3 */
  {61, &PORTC, 1, 0},		/* C#3 */
  {62, &PORTC, 2, 0},		/* D3 */
  {63, &PORTC, 3, 0},		/* D#3 */
  {64, &PORTC, 4, 0},		/* E3 */
  {65, &PORTC, 5, 0},		/* F3 */
  {66, &PORTB, 0, 0},		/* F#3 */
  {67, &PORTD, 1, 0},		/* G3 */
  {68, &PORTD, 6, 0},		/* G#3 */
  {69, &PORTD, 5, 0},		/* A3 */
  {70, &PORTB, 2, 0},		/* A#3 */
  {71, &PORTB, 3, 0},		/* B3 */
  {72, &PORTD, 3, 0},		/* C4 */
  {73, &PORTD, 4, 0},		/* C#4 */
  {74, &PORTD, 7, 0}		/* D4 */
};

/* mapping table for input port and note (Config 3, 5, 6) */
struct portInfo_t portInfoIn []= {
  {72, &PIND, 3, 0},		/* C4   (MODE 3, 5  6) */
  {70, &PINB, 2, 0},		/* A#3  (MODE 3, 5, 6) */
  {71, &PINB, 3, 0},		/* B3   (MODE 3, 5, 6) */
  {69, &PIND, 5, 0},		/* A3   (MODE 3, 5, 6) */
  {59, &PINB, 4, 0},		/* B2 for built-in button (MODE *) */
  {60, &PINC, 0, 0},		/* C3   (MODE    5, 6) */
  {61, &PINC, 1, 0},		/* C#3  (MODE    5, 6) */
  {62, &PINC, 2, 0},		/* D3   (MODE    5, 6) */
  {63, &PINC, 3, 0},		/* D#3  (MODE    5, 6) */
  {64, &PINC, 4, 0},		/* E3   (MODE    5, 6) */
  {65, &PINC, 5, 0},		/* F3   (MODE    5, 6) */
  {66, &PINB, 0, 0},		/* F#3  (MODE    5, 6) */
  {67, &PIND, 1, 0},		/* G3   (MODE    5, 6) */
  {68, &PIND, 6, 0},		/* G#3  (MODE    5, 6) */
  {73, &PIND, 4, 0},		/* C#4  (MODE    5) */
  {74, &PIND, 7, 0},		/* D4   (MODE    5) */
};


/* MONO出力用のポートのマッピング */
struct monoMap_t {
  volatile uchar *port;
  uchar bit;
} monoMap [] = {
  {&PORTD, 6},			/* BIT0 */
  {&PORTD, 5},			/* BIT1 */
  {&PORTB, 2},			/* BIT2 */
  {&PORTB, 3},			/* BIT3 */
  {&PORTD, 3},			/* BIT4 */
  {&PORTD, 4},			/* BIT5 */
  {&PORTD, 7},			/* BIT6 */
};


/* テーブル参照用 */
uchar polyOutMin = 60;
uchar polyOutMax = 67;

uchar scanKeyMin = 0;
uchar scanKeyMax = 4;

/*--------- MONOME MODE KEY SCAN用 ------------- */
struct scanPortInfo_t {
  volatile uchar *port;
  uchar bit;
  volatile uchar *ddr;
  volatile uchar *in;
};

#define SCAN_ROW_SIZE 8
#define SCAN_COL_SIZE 8

struct scanPortInfo_t scanCol []= {
  {&PORTD, 6, &DDRD, &PIND},	/* AOUT-0 */
  {&PORTD, 5, &DDRD, &PIND},	/* AOUT-1 */
  {&PORTB, 1, &DDRB, &PINB},	/* AOUT-2 */
  {&PORTB, 2, &DDRB, &PINB},	/* AOUT-3 */
  {&PORTB, 3, &DDRB, &PINB},	/* DOUT-0 */
  {&PORTD, 3, &DDRD, &PIND},	/* DOUT-1 */
  {&PORTD, 4, &DDRD, &PIND},	/* DOUT-2 */
  {&PORTD, 7, &DDRD, &PIND},	/* DOUT-3 */
};

struct scanPortInfo_t scanRow []= {
  {&PORTC, 0, &DDRC, &PINC},	/* AIN-0 */
  {&PORTC, 1, &DDRC, &PINC},	/* AIN-1 */
  {&PORTC, 2, &DDRC, &PINC},	/* AIN-2 */
  {&PORTC, 3, &DDRC, &PINC},	/* AIN-3 */
  {&PORTC, 4, &DDRC, &PINC},	/* DIN-0 */
  {&PORTC, 5, &DDRC, &PINC},	/* DIN-1 */
  {&PORTB, 0, &DDRB, &PINB},	/* DIN-2 */
  {&PORTD, 1, &DDRD, &PIND},	/* DIN-3 */
};

/*--------- MONOME MODE KEY SCAN用 ------------- */

/* インプリされたモードのチェック */
#define MODE_CHECK(c) (((c) < 4) || ((c) == 5) || ((c) == 6) || ((c) == 7))

/*----------------------------------------------- */
/* I/O入出力LEDの処理 */

#define SET_LED(val) setPortBit(&PORTB, 5, (val))

/* timing clock support */
#define CLOCK_OUT() { if (conf == 3) {flipPortBit(&PORTD, 6); } }

/* polyOutのあるモード */
#define POLY_OUT_CHECK() (conf < 4)

/* trigOutのあるモード */
#define TRIG_OUT_CHECK() (conf == 3)

void monoOut(uchar note, uchar val) {
  uchar i;
  if (conf == 1) {		/* OLNY MODE1 */
    for (i = 0; i < 7; i++) {
      setPortBit(monoMap[i].port, monoMap[i].bit, (note & _BV(i)));
    }
  }
}

/* for ToneOut */
#define SND_OUT_CHECK() ((conf != 6) && (conf != 7))
#define SND_ON  {OCR1A=getTp(note); DDRB |=  0b00000010; } /* PB1=out */
#define SND_OFF {DDRB &= ~0b00000010; } /* PB1=in */

/* key input */
#define MAX_KEYS (SCAN_ROW_SIZE * SCAN_COL_SIZE)
static uchar keyCountButton[MAX_KEYS];
static uchar buttonStatus[MAX_KEYS];
/* matrix LED status */
static uchar matrixLedStatus[MAX_KEYS];

#define SCAN_KEY_CHECK() (conf != 7)

/* PB1 == H (COHRD MODE) L (NORMAL MODE) */
#define CHORD_MODE_CHECK(p) (((p) & 0x02) == 0)

/* PD7 == H (NORMAL) L (MINOR CHORD) */
#define CHORD_SHIFT_MINOR(p) (((p) & 0x80) == 0)

/* PD4 == H (NORMAL) L (7th CHORD) */
#define CHORD_SHIFT_7TH(p) (((p) & 0x10) == 0)

#define KEY_MODE_READ(x) {\
    if (conf == 6) {\
      x = x | (PINB & 0x02) | (PIND & 0x80) | (PIND & 0x10); \
    }\
}

#define KEY_EVENT_SEND(note, x) { \
    uchar noteOnOff; \
    noteOnOff = (((x) & 1) == 0) ? 1 : 0; \
    if (conf != 6) { \
      keyEventAppend((note), noteOnOff);	\
    } else { \
      if (CHORD_MODE_CHECK(x)) { \
	keyEventAppend((note), noteOnOff); \
      } else {\
	if (CHORD_SHIFT_MINOR(x)) { \
	  if (CHORD_SHIFT_7TH(x)) {  \
	    keyEventAppend((note), noteOnOff);  \
	    keyEventAppend((note) + 3, noteOnOff);  \
	    keyEventAppend((note) + 7, noteOnOff);  \
	    keyEventAppend((note) + 10, noteOnOff);  \
	  } else { \
	    keyEventAppend((note), noteOnOff); \
	    keyEventAppend((note) + 3, noteOnOff);  \
	    keyEventAppend((note) + 7, noteOnOff);  \
	  } \
	} else if (CHORD_SHIFT_7TH(x)) { \
	  keyEventAppend((note), noteOnOff); \
	  keyEventAppend((note) + 4, noteOnOff); \
	  keyEventAppend((note) + 7, noteOnOff); \
	  keyEventAppend((note) + 10, noteOnOff); \
	} else { \
	  keyEventAppend((note), noteOnOff); \
	  keyEventAppend((note) + 4, noteOnOff); \
	  keyEventAppend((note) + 7, noteOnOff); \
	}\
      } \
    } \
  }

#define SCAN_MATRIX_CHECK() (conf == 7)

void portInit(uchar cf) {
  /* setup PORTB */
  DDRB &= ~portConfig[cf].ddrbIn;
  DDRB |= portConfig[cf].ddrbOut;
  PORTB |= portConfig[cf].ddrbIn;  /* pull-up all input ports */
  PORTB &= ~portConfig[cf].ddrbOut; /* PUTPUT 0 for all output pin */

  /* setup PORTC */
  DDRC &= ~portConfig[cf].ddrcIn;
  DDRC |= portConfig[cf].ddrcOut;
  PORTC |= portConfig[cf].ddrcIn;   /* pull-up all input ports*/
  PORTC &= ~portConfig[cf].ddrcOut; /* PUTPUT 0 for all output pin */


  /* setup PORTD */
  DDRD &= ~portConfig[cf].ddrdIn;
  DDRD |= portConfig[cf].ddrdOut;
  PORTD |= portConfig[cf].ddrdIn;   /* pull-up all input ports */
  PORTD &= ~portConfig[cf].ddrdOut; /* PUTPUT 0 for all output pin */

  /* Timer 1を初期化する */
  if (conf != 7) {
    TCCR1A = 0b01000000; /* CTC mode toggle OC1A, OC1B no use, mode = 4 */
    TCCR1B = 0b00001011; /* 1/64 clock */
    TCCR1C = 0b00000000;
    OCR1A = 0;
  } else {
    TCCR1A = 0b00000000; /* NORMAL PORT OPERATION*/
    TCCR1B = 0b00000000; /* no clock source */
    TCCR1C = 0b00000000;
    OCR1A = 0;
  }

  /* キースキャンテーブルの初期化 */
  initializeKeyStatus();

  /* テーブル参照用 */
  polyOutMin = 60;
  polyOutMax = polyOutMin + ((conf == 2) ? 15 : 8); /* 75: MODE=1, 68 : MODE=2,3 */

  /* reference for portInfoIn */
  if (conf == 3) {
    scanKeyMin = 0;
    scanKeyMax = 5;
  } else if (conf == 5) {
    scanKeyMin = 0;
    scanKeyMax = 16;
  } else if (conf == 6) {
    scanKeyMin = 0;
    scanKeyMax = 14;
  } else {			/* CONFIG 1, 2, (4), 7 */
    scanKeyMin = 4;
    scanKeyMax = 5;
  }
  polyOutInit();
}

/* EOF */
